/*
 * ServerObjectHandler.h, part of VCMI engine
 *
 * Authors: listed in file AUTHORS in main folder
 *
 * License: GNU General Public License v2.0 or later
 * Full text of license available in license.txt file, in main folder
 *
 */
 
#pragma once
class CGObjectInstance;
class CGHeroInstance;
class BattleResult;
class CStackBasicDescriptor;
class IGameCallback;

//packets
struct InfoWindow;

namespace server_objects{


/**
* Dispatches events to handlers
*/
class IEventDispatcher
{
public:
	virtual ~IEventDispatcher();
	virtual void onHeroVisit(const CGObjectInstance *o, const CGHeroInstance *visitor) = 0;
	
	/**
	* Factory function
	* @param cb reference to GameHandler
	* @return Concrete instance of dispatecher
	*/
	static IEventDispatcher * createDispatcher(IGameCallback * cb);
protected:
	IEventDispatcher();
};



}//namespace server_objects

namespace hero_mechanics {

/**
 * Calculates what creatures and how many to be raised from a battle.
 * @param hero the necromant
 * @param battleResult the results of the battle.
 * @return Returns a pair with the first value indicating the ID of the creature
 * type and second value the amount. Both values are returned as -1 if necromancy
 * could not be applied.
 */
CStackBasicDescriptor calculateNecromancy (const CGHeroInstance *hero, const BattleResult &battleResult);

/**
 * Show the necromancy dialog with information about units raised.
 * @param hero the necromant
 * @param raisedStack Pair where the first element represents ID of the raised creature
 * and the second element the amount.
 * @deprecated 
 */
void showNecromancyDialog(const CGHeroInstance *hero, const CStackBasicDescriptor &raisedStack);

/**
 * Sets up the necromancy dialog with information about units raised.
 * @param hero the necromant
 * @param raisedStack Pair where the first element represents ID of the raised creature
 * and the second element the amount.
 * @param iw Info window packet
 */
void prepareNecromancyDialog(const CGHeroInstance *hero, const CStackBasicDescriptor &raisedStack, InfoWindow &iw);

} //namespace hero_mechanics 